/*******************************************************************************
 * Copyright (c) 2016, 2017 Obeo.
 * This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *    Obeo - initial API and implementation
 *******************************************************************************/
package org.eclipse.sirius.properties.core.api;

import java.util.Map;

import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.sirius.viewpoint.description.tool.InitialOperation;
import org.eclipse.sirius.viewpoint.description.tool.ToolPackage;

/**
 * The default converter to use for descriptions with an initial operation.
 * 
 * @author sbegaudeau
 * 
 * @param <SIRIUS>
 *            The type of the Sirius EObject
 */
public class DefaultDescriptionWithInitialOperationConverter<SIRIUS extends EObject> extends DefaultDescriptionConverter<SIRIUS> {

    /**
     * The EAttribute containing the expression which will be used to run the
     * initial operation.
     */
    private EAttribute initialOperationExpressionEAttribute;

    /**
     * The constructor.
     * 
     * @param siriusClass
     *            The class of the Sirius EObject
     * @param eefEClass
     *            The EClass of the EEF EObject
     * @param initialOperationExpressionEAttribute
     *            The EAttribute containing the expression which will be used to
     *            run the initial operation.
     */
    public DefaultDescriptionWithInitialOperationConverter(Class<SIRIUS> siriusClass, EClass eefEClass, EAttribute initialOperationExpressionEAttribute) {
        super(siriusClass, eefEClass);
        this.initialOperationExpressionEAttribute = initialOperationExpressionEAttribute;
    }

    @Override
    protected void convertEReference(SIRIUS siriusEObject, EObject eefEObject, EReference eReference, Map<String, Object> parameters, TransformationCache cache) {
        if (eReference.getEReferenceType().equals(ToolPackage.Literals.INITIAL_OPERATION)) {
            Object initialOperation = siriusEObject.eGet(eReference);
            if (initialOperation instanceof InitialOperation) {
                String initialOperationExpression = this.getExpressionForOperation((InitialOperation) initialOperation);
                eefEObject.eSet(this.initialOperationExpressionEAttribute, initialOperationExpression);
                return;
            }
        }
        super.convertEReference(siriusEObject, eefEObject, eReference, parameters, cache);
    }

}
